import yt
import matplotlib.cm as cm

# Load the dataset.
ds = yt.load("IsolatedGalaxy/galaxy0030/galaxy0030")

# Create projections using each colormap available.
p = yt.ProjectionPlot(ds, "z", "density", weight_field = "density", width=0.4)

for cmap in cm.datad:
    if cmap.startswith("idl"):
        continue
    p.set_cmap(field="density", cmap=cmap)
    p.annotate_title(cmap)
    p.save('Projection_%s.png' % cmap.replace(' ', '_'))



import yt

# first add density contours on a density slice
ds = yt.load("GasSloshing/sloshing_nomag2_hdf5_plt_cnt_0150")

# add density contours on the density slice.
p = yt.SlicePlot(ds, "x", "density")
p.annotate_contour("density")
p.save()

# then add temperature contours on the same density slice
p = yt.SlicePlot(ds, "x", "density")
p.annotate_contour("temperature")
p.save(str(ds)+'_T_contour')



# This extension is quite simple:
#  1. It accepts a script name
#  2. This script is added to the document in a literalinclude
#  3. Any _static images found will be added

from sphinx.util.compat import Directive
from docutils.parsers.rst import directives
import os, glob, shutil

# Some of this magic comes from the matplotlib plot_directive.

def setup(app):
    app.add_directive('yt_colormaps', ColormapScript)
    setup.app = app
    setup.config = app.config
    setup.confdir = app.confdir

    retdict = dict(
        version='0.1',
        parallel_read_safe=True,
        parallel_write_safe=True
    )

    return retdict

class ColormapScript(Directive):
    required_arguments = 1
    optional_arguments = 0

    def run(self):
        rst_file = self.state_machine.document.attributes['source']
        rst_dir = os.path.abspath(os.path.dirname(rst_file))
        script_fn = directives.path(self.arguments[0])
        script_bn = os.path.basename(script_fn)
        script_name = os.path.basename(self.arguments[0]).split(".")[0]

        # This magic is from matplotlib
        dest_dir = os.path.abspath(os.path.join(setup.app.builder.outdir,
                                                os.path.dirname(script_fn)))
        if not os.path.exists(dest_dir):
            os.makedirs(dest_dir) # no problem here for me, but just use built-ins

        rel_dir = os.path.relpath(rst_dir, setup.confdir)
        place = os.path.join(dest_dir, rel_dir)
        if not os.path.isdir(place): os.makedirs(place)
        shutil.copyfile(os.path.join(rst_dir, script_fn),
                        os.path.join(place, script_bn))

        im_path = os.path.join(rst_dir, "_static")
        images = sorted(glob.glob(os.path.join(im_path, "*.png")))
        lines = []
        for im in images:
            im_name = os.path.join("_static", os.path.basename(im))
            lines.append(".. image:: %s" % im_name)
            lines.append("   :width: 400")
            lines.append("   :target: ../../_images/%s" % os.path.basename(im))
            lines.append("\n")
        lines.append("\n")
        self.state_machine.insert_input(lines, rst_file)
        return []



import yt
import yt.units as u

ds = yt.load('HiresIsolatedGalaxy/DD0044/DD0044')

center = [0.53, 0.53, 0.53]
normal = [0,0,1]
radius = 40*u.kpc
height = 2*u.kpc

disk = ds.disk(center, [0,0,1], radius, height)

profile = yt.create_profile(
    data_source=disk,
    bin_fields=["radius", "cylindrical_tangential_velocity"],
    fields=["cell_mass"],
    n_bins=256,
    units=dict(radius="kpc",
               cylindrical_tangential_velocity="km/s",
               cell_mass="Msun"),
    logs=dict(radius=False,
              cylindrical_tangential_velocity=False),
    weight_field=None,
    extrema=dict(radius=(0,40),
                 cylindrical_tangential_velocity=(-250, 250)),
    )

plot = yt.PhasePlot.from_profile(profile)
plot.set_cmap("cell_mass", "YlOrRd")

plot.save()



import yt

# Load the dataset.
ds = yt.load("GalaxyClusterMerger/fiducial_1to3_b0.273d_hdf5_plt_cnt_0175")

# Create a data object that represents the whole box.
ad = ds.all_data()

# This is identical to the simple phase plot, except we supply 
# the fractional=True keyword to divide the profile data by the sum. 
plot = yt.PhasePlot(ad, "density", "temperature", "cell_mass",
                    weight_field=None, fractional=True)

# Save the image.
# Optionally, give a string as an argument
# to name files with a keyword.
plot.save()



import yt

ds = yt.load("enzo_tiny_cosmology/DD0046/DD0046")
p = yt.ProjectionPlot(ds, "z", "density")
p.annotate_sphere([0.54,0.72], radius=(1, 'Mpc'), coord_system='axis', text='Halo #7')
p.annotate_sphere([0.65,0.38,0.3], radius=(1.5, 'Mpc'), coord_system='data', circle_args={'color':'green', 'linewidth':4, 'linestyle':'dashed'})
p.annotate_arrow([0.87,0.59,0.2], coord_system='data', plot_args={'color':'red'})
p.annotate_text([10,20], 'Some halos', coord_system='plot')
p.annotate_marker([0.45,0.1,0.4], coord_system='data',
                  plot_args={'color':'yellow', 's':500})
p.annotate_line([0.2,0.4], [0.3,0.9], coord_system='axis')
p.annotate_timestamp(redshift=True)
p.annotate_scale()
p.save()



# You must run this job in parallel.
# There are several mpi flags which can be useful in order for it to work OK.
# It requires at least 3 processors in order to run because of the way in which
# rockstar divides up the work.  Make sure you have mpi4py installed as per
# http://yt-project.org/docs/dev/analyzing/parallel_computation.html#setting-up-parallel-yt

# Usage: mpirun -np <num_procs> --mca btl ^openib python this_script.py

import yt
from yt.analysis_modules.halo_analysis.halo_catalog import HaloCatalog
from yt.data_objects.particle_filters import add_particle_filter
from yt.analysis_modules.halo_finding.rockstar.api import RockstarHaloFinder
yt.enable_parallelism() # rockstar halofinding requires parallelism

# Create a dark matter particle filter
# This will be code dependent, but this function here is true for enzo

def DarkMatter(pfilter, data):
    filter = data[("all", "particle_type")] == 1 # DM = 1, Stars = 2
    return filter

add_particle_filter("dark_matter", function=DarkMatter, filtered_type='all', \
                    requires=["particle_type"])

# First, we make sure that this script is being run using mpirun with
# at least 3 processors as indicated in the comments above.
assert(yt.communication_system.communicators[-1].size >= 3)

# Load the dataset and apply dark matter filter
fn = "Enzo_64/DD0043/data0043"
ds = yt.load(fn)
ds.add_particle_filter('dark_matter')

# Determine highest resolution DM particle mass in sim by looking
# at the extrema of the dark_matter particle_mass field.
ad = ds.all_data()
min_dm_mass = ad.quantities.extrema(('dark_matter','particle_mass'))[0]

# Define a new particle filter to isolate all highest resolution DM particles
# and apply it to dataset
def MaxResDarkMatter(pfilter, data):
    return data["particle_mass"] <= 1.01 * min_dm_mass

add_particle_filter("max_res_dark_matter", function=MaxResDarkMatter, \
                    filtered_type='dark_matter', requires=["particle_mass"])
ds.add_particle_filter('max_res_dark_matter')

# If desired, we can see the total number of DM and High-res DM particles
#if yt.is_root():
#    print("Simulation has %d DM particles." %
#          ad['dark_matter','particle_type'].shape)
#    print("Simulation has %d Highest Res DM particles." %
#          ad['max_res_dark_matter', 'particle_type'].shape)

# Run the halo catalog on the dataset only on the highest resolution dark matter
# particles
hc = HaloCatalog(data_ds=ds, finder_method='rockstar', \
                 finder_kwargs={'dm_only':True, 'particle_type':'max_res_dark_matter'})
hc.create()

# Or alternatively, just run the RockstarHaloFinder and later import the
# output file as necessary.  You can skip this step if you've already run it
# once, but be careful since subsequent halo finds will overwrite this data.
#rhf = RockstarHaloFinder(ds, particle_type="max_res_dark_matter")
#rhf.run()
# Load the halo list from a rockstar output for this dataset
# Create a projection with the halos overplot on top
#halos = yt.load('rockstar_halos/halos_0.0.bin')
#hc = HaloCatalog(halos_ds=halos)
#hc.load()

# Regardless of your method of creating the halo catalog, use it to overplot the
# halos on a projection.
p = yt.ProjectionPlot(ds, "x", "density")
p.annotate_halos(hc, annotate_field = 'particle_identifier', width=(10,'Mpc'), factor=2)
p.save()



import yt

ts = yt.load("enzo_tiny_cosmology/DD000?/DD000?")
for ds in ts:
    p = yt.ProjectionPlot(ds, "z", "density")
    p.annotate_timestamp(corner='upper_left', redshift=True, draw_inset_box=True)
    p.annotate_scale(corner='upper_right')
    p.save()



import os
from pkg_resources import resource_filename
import shutil
from subprocess import Popen, PIPE
import sys
import tempfile


def check_for_openmp():
    """Returns True if local setup supports OpenMP, False otherwise"""

    # See https://bugs.python.org/issue25150
    if sys.version_info[:3] == (3, 5, 0) or os.name == 'nt':
        return False

    # Create a temporary directory
    tmpdir = tempfile.mkdtemp()
    curdir = os.getcwd()
    exit_code = 1

    try:
        os.chdir(tmpdir)

        # Get compiler invocation
        compiler = os.getenv('CC', 'cc')
        compiler = compiler.split(' ')

        # Attempt to compile a test script.
        # See http://openmp.org/wp/openmp-compilers/
        filename = r'test.c'
        file = open(filename, 'wt', 1)
        file.write(
            "#include <omp.h>\n"
            "#include <stdio.h>\n"
            "int main() {\n"
            "#pragma omp parallel\n"
            "printf(\"Hello from thread %d, nthreads %d\\n\", omp_get_thread_num(), omp_get_num_threads());\n"
            "}"
        )
        file.flush()
        p = Popen(compiler + ['-fopenmp', filename],
                  stdin=PIPE, stdout=PIPE, stderr=PIPE)
        output, err = p.communicate()
        exit_code = p.returncode
        
        if exit_code != 0:
            print("Compilation of OpenMP test code failed with the error: ")
            print(err.decode('utf8'))
            print("Disabling OpenMP support. ")

        # Clean up
        file.close()
    except OSError:
        print("check_for_openmp() could not find your C compiler. "
              "Attempted to use '%s'. " % compiler)
        return False
    finally:
        os.chdir(curdir)
        shutil.rmtree(tmpdir)

    return exit_code == 0


def check_for_pyembree():
    try:
        fn = resource_filename("pyembree", "rtcore.pxd")
    except ImportError:
        return None
    return os.path.dirname(fn)

def in_conda_env():
    return any(s in sys.version for s in ("Anaconda", "Continuum"))

def read_embree_location():
    '''

    Attempts to locate the embree installation. First, we check for an
    EMBREE_DIR environment variable. If one is not defined, we look for
    an embree.cfg file in the root yt source directory. Finally, if that
    is not present, we default to /usr/local. If embree is installed in a
    non-standard location and none of the above are set, the compile will
    not succeed. This only gets called if check_for_pyembree() returns
    something other than None.

    '''

    rd = os.environ.get('EMBREE_DIR')
    if rd is None:
        try:
            rd = open("embree.cfg").read().strip()
        except IOError:
            rd = '/usr/local'

    fail_msg = ("I attempted to find Embree headers in %s. \n"
               "If this is not correct, please set your correct embree location \n"
               "using EMBREE_DIR environment variable or your embree.cfg file. \n"
               "Please see http://yt-project.org/docs/dev/visualizing/unstructured_mesh_rendering.html "
                "for more information. \n" % rd)

    # Create a temporary directory
    tmpdir = tempfile.mkdtemp()
    curdir = os.getcwd()

    try:
        os.chdir(tmpdir)

        # Get compiler invocation
        compiler = os.getenv('CXX', 'c++')
        compiler = compiler.split(' ')

        # Attempt to compile a test script.
        filename = r'test.cpp'
        file = open(filename, 'wt', 1)
        file.write(
            '#include "embree2/rtcore.h"\n'
            'int main() {\n'
            'return 0;\n'
            '}'
        )
        file.flush()
        p = Popen(compiler + ['-I%s/include/' % rd, filename], 
                  stdin=PIPE, stdout=PIPE, stderr=PIPE)
        output, err = p.communicate()
        exit_code = p.returncode

        if exit_code != 0:
            print("Pyembree is installed, but I could not compile Embree test code.")
            print("The error message was: ")
            print(err)
            print(fail_msg)

        # Clean up
        file.close()

    except OSError:
        print("read_embree_location() could not find your C compiler. "
              "Attempted to use '%s'. " % compiler)
        return False

    finally:
        os.chdir(curdir)
        shutil.rmtree(tmpdir)

    return rd



import yt
import numpy as np
import matplotlib.pylab as pl

from yt.visualization.api import Streamlines
from yt.units import Mpc
from mpl_toolkits.mplot3d import Axes3D

# Load the dataset
ds = yt.load('IsolatedGalaxy/galaxy0030/galaxy0030')

# Define c: the center of the box, N: the number of streamlines,
# scale: the spatial scale of the streamlines relative to the boxsize,
# and then pos: the random positions of the streamlines.
c = ds.domain_center
N = 100
scale = ds.domain_width[0]
pos_dx = np.random.random((N,3))*scale-scale/2.
pos = c+pos_dx

# Create streamlines of the 3D vector velocity and integrate them through
# the box defined above
streamlines = Streamlines(ds, pos, 'velocity_x', 'velocity_y', 'velocity_z',
                          length=1.0*Mpc, get_magnitude=True)
streamlines.integrate_through_volume()

# Create a 3D plot, trace the streamlines through the 3D volume of the plot
fig=pl.figure()
ax = Axes3D(fig)
for stream in streamlines.streamlines:
    stream = stream[np.all(stream != 0.0, axis=1)]
    ax.plot3D(stream[:,0], stream[:,1], stream[:,2], alpha=0.1)

# Save the plot to disk.
pl.savefig('streamlines.png')



import yt

# Load a Gadget dataset following the demonstration notebook.
fname = 'GadgetDiskGalaxy/snapshot_200.hdf5'

unit_base = {'UnitLength_in_cm'         : 3.08568e+21,
             'UnitMass_in_g'            :   1.989e+43,
             'UnitVelocity_in_cm_per_s' :      100000}

bbox_lim = 1e5  # kpc

bbox = [[-bbox_lim, bbox_lim],
        [-bbox_lim, bbox_lim],
        [-bbox_lim, bbox_lim]]

ds = yt.load(fname, unit_base=unit_base, bounding_box=bbox)

# Create a derived field, the metal density.
def _metal_density(field, data):
    density = data['PartType0', 'Density']
    Z = data['PartType0', 'metallicity']
    return density * Z

# Add it to the dataset.
ds.add_field(('PartType0', 'metal_density'), function=_metal_density,
             units="g/cm**3", particle_type=True)


# Add the corresponding smoothed field to the dataset.
from yt.fields.particle_fields import add_volume_weighted_smoothed_field

add_volume_weighted_smoothed_field('PartType0', 'Coordinates', 'Masses',
                                   'SmoothingLength', 'Density',
                                   'metal_density', ds.field_info)

# Define the region where the disk galaxy is. (See the Gadget notebook for
# details. Here I make the box a little larger than needed to eliminate the
# margin effect.)
center = ds.arr([31996, 31474, 28970], "code_length")
box_size = ds.quan(250, "code_length")
left_edge = center - box_size/2*1.1
right_edge = center + box_size/2*1.1
box = ds.box(left_edge=left_edge, right_edge=right_edge)

# And make a projection plot!
yt.ProjectionPlot(ds, 'z',
                  ('deposit', 'PartType0_smoothed_metal_density'),
                  center=center, width=box_size, data_source=box).save()



contents = [
  ("Getting Started", [
     ("welcome/index.html", "Welcome to yt!",
      "What's yt all about?"),
     ("orientation/index.html", "yt Orientation",
      "Quickly get up and running with yt: zero to sixty."),
     ("help/index.html", "How to Ask for Help",
      "Some guidelines on how and where to ask for help with yt"),
     ("workshop.html", "Workshop Tutorials",
      "Videos, slides and scripts from the 2012 workshop covering many " +
      "aspects of yt, from beginning to advanced."),
  ]),
  ("Everyday yt", [
     ("analyzing/index.html", "Analyzing Data",
      "An overview of different ways to handle and process data: loading " +
      "data, using and manipulating objects and fields, examining and " +
      "manipulating particles, derived fields, generating processed data, " +
      "time series analysis."),
     ("visualizing/index.html", "Visualizing Data",
      "An overview of different ways to visualize data: making projections, " +
      "slices, phase plots, streamlines, and volume rendering; modifying " +
      "plots; the fixed resolution buffer."),
     ("interacting/index.html", "Interacting with yt",
      "Different ways -- scripting, GUIs, prompts, explorers -- to explore " +
      "your data."),
     ("analysis_modules/index.html", "Analysis Modules",
      "Discussions of some provided procedures for astrophysical analysis " +
      "like halo finding and synthetic spectra.  Halo finding, analyzing " +
      "cosmology simulations, halo mass functions, halo profiling, light " +
      "cone generator, making absorption spectrums, star particle " +
      "analysis, two-point functions, halo merger trees, clump finding, " +
      "radial column density, exporting to sunrise.")
  ]),
  ("Advanced Usage", [
     ("advanced/index.html", "Advanced yt usage",
      "Advanced topics: parallelism, debugging, ways to drive yt, " +
      "developing"),
     ("getting_involved/index.html", "Getting Involved",
      "How to participate in the community, contribute code and share " +
      "scripts"),
  ]),
  ("Reference Materials", [
     ("cookbook/index.html", "The Cookbook",
      "A bunch of illustrated examples of how to do things"),
     ("reference/index.html", "Reference Materials",
      "A list of all bundled fields, API documentation, the Change Log..."),
     ("faq/index.html", "FAQ",
      "Frequently Asked Questions: answered for you!")
  ]),
]

heading_template = r"""
<h2>%s</h2>
<table class="contentstable" align="center">
%s
</table>
"""

subheading_template = r"""
  <tr valign="top">
    <td width="25%%">
      <p>
        <a href="%s">%s</a>
      </p>
    </td>
    <td width="75%%">
      <p class="linkdescr">%s</p>
    </td>
  </tr>
"""

t = ""
for heading, items in contents:
    s = ""
    for subheading in items:
        s += subheading_template % subheading
    t += heading_template % (heading, s)
print(t)



import os
import yt
from yt.analysis_modules.cosmological_observation.api import \
    LightRay

ds = yt.load("IsolatedGalaxy/galaxy0030/galaxy0030")
lr = LightRay(ds)

# With a single dataset, a start_position and
# end_position or trajectory must be given.
# These positions can be defined as xyz coordinates,
# but here we just use the two opposite corners of the 
# simulation box.  Alternatively, trajectory should 
# be given as (r, theta, phi)
lr.make_light_ray(start_position=ds.domain_left_edge,
                  end_position=ds.domain_right_edge,
                  solution_filename='lightraysolution.txt',
                  data_filename='lightray.h5',
                  fields=['temperature', 'density'])

# Optionally, we can now overplot this ray on a projection of the source
# dataset
p = yt.ProjectionPlot(ds, 'z', 'density')
p.annotate_ray(lr)
p.save()



import yt
from yt.analysis_modules.halo_analysis.api import HaloCatalog

# Load the data set with the full simulation information
# and rockstar halos
data_ds = yt.load('Enzo_64/RD0006/RedshiftOutput0006')
halos_ds = yt.load('rockstar_halos/halos_0.0.bin')

# Instantiate a catalog using those two paramter files
hc = HaloCatalog(data_ds=data_ds, halos_ds=halos_ds)

# Filter out less massive halos
hc.add_filter("quantity_value", "particle_mass", ">", 1e14, "Msun")

# This recipe creates a spherical data container, computes
# radial profiles, and calculates r_200 and M_200.
hc.add_recipe("calculate_virial_quantities", ["radius", "matter_mass"])

# Create a sphere container with radius 5x r_200.
field_params = dict(virial_radius=('quantity', 'radius_200'))
hc.add_callback('sphere', radius_field='radius_200', factor=5,
                field_parameters=field_params)

# Compute profiles of T vs. r/r_200
hc.add_callback('profile', ['virial_radius_fraction'],
                [('gas', 'temperature')],
                storage='virial_profiles',
                weight_field='cell_mass',
                accumulation=False, output_dir='profiles')

# Save the profiles
hc.add_callback("save_profiles", storage="virial_profiles",
                output_dir="profiles")

hc.create()



import yt
import numpy as np
from yt.visualization.api import Streamlines
import matplotlib.pylab as pl
from mpl_toolkits.mplot3d import Axes3D
from mpl_toolkits.mplot3d.art3d import Poly3DCollection

# Load the dataset
ds = yt.load("IsolatedGalaxy/galaxy0030/galaxy0030")

# Define c: the center of the box, N: the number of streamlines,
# scale: the spatial scale of the streamlines relative to the boxsize,
# and then pos: the random positions of the streamlines.
c = ds.arr([0.5]*3, 'code_length')
N = 30
scale = ds.quan(15, 'kpc').in_units('code_length')  # 15 kpc in code units
pos_dx = np.random.random((N,3))*scale-scale/2.
pos = c+pos_dx

# Create the streamlines from these positions with the velocity fields as the
# fields to be traced
streamlines = Streamlines(ds, pos, 'velocity_x', 'velocity_y', 'velocity_z', length=1.0)
streamlines.integrate_through_volume()

# Create a 3D matplotlib figure for visualizing the streamlines
fig=pl.figure()
ax = Axes3D(fig)

# Trace the streamlines through the volume of the 3D figure
for stream in streamlines.streamlines:
    stream = stream[np.all(stream != 0.0, axis=1)]

    # Make the colors of each stream vary continuously from blue to red
    # from low-x to high-x of the stream start position (each color is R, G, B)
    # can omit and just set streamline colors to a fixed color
    x_start_pos = ds.arr(stream[0,0], 'code_length')
    x_start_pos -= ds.arr(0.5, 'code_length')
    x_start_pos /= scale
    x_start_pos += 0.5
    color = np.array([x_start_pos, 0, 1-x_start_pos])

    # Plot the stream in 3D
    ax.plot3D(stream[:,0], stream[:,1], stream[:,2], alpha=0.3, color=color)

# Create a sphere object centered on the highest density point in the simulation
# with radius = 1 Mpc
sphere = ds.sphere("max", (1.0, "Mpc"))

# Identify the isodensity surface in this sphere with density = 1e-24 g/cm^3
surface = ds.surface(sphere, "density", 1e-24)

# Color this isodensity surface according to the log of the temperature field
colors = yt.apply_colormap(np.log10(surface["temperature"]), cmap_name="hot")

# Render this surface
p3dc = Poly3DCollection(surface.triangles, linewidth=0.0)
colors = colors[0,:,:]/255.  # scale to [0,1]
colors[:,3] = 0.3            # alpha = 0.3
p3dc.set_facecolors(colors)
ax.add_collection(p3dc)

# Save the figure
pl.savefig('streamlines_isocontour.png')



#!/usr/bin/python

import os
import glob
import sys
import shutil
import tempfile
import traceback
import subprocess
import matplotlib
matplotlib.use('Agg')
from multiprocessing import Pool
from yt.config import ytcfg

FPATTERNS = ['*.png', '*.txt', '*.h5', '*.dat', '*.mp4']
DPATTERNS = ['LC*', 'LR', 'DD0046']
BADF = ['cloudy_emissivity.h5', 'apec_emissivity.h5',
        'xray_emissivity.h5', 'AMRGridData_Slice_x_density.png']
CWD = os.getcwd()
ytcfg["yt", "serialize"] = "False"
PARALLEL_TEST = {"rockstar_nest": "3"}
BLACKLIST = ["opengl_ipython", "opengl_vr"]


def prep_dirs():
    for directory in glob.glob('%s/*' % ytcfg.get("yt", "test_data_dir")):
        os.symlink(directory, os.path.basename(directory))


def run_recipe(payload):
    recipe, = payload
    module_name, ext = os.path.splitext(os.path.basename(recipe))
    dest = os.path.join(os.path.dirname(recipe), '_static', module_name)
    if module_name in BLACKLIST:
        return 0
    if not os.path.exists("%s/_temp/%s.done" % (CWD, module_name)):
        sys.stderr.write('Started %s\n' % module_name)
        tmpdir = tempfile.mkdtemp()
        os.chdir(tmpdir)
        prep_dirs()
        if module_name in PARALLEL_TEST:
            cmd = ["mpiexec", "-n", PARALLEL_TEST[module_name],
                   "python2", recipe]
        else:
            cmd = ["python", recipe]
        try:
            subprocess.check_call(cmd)
        except:
            trace = "".join(traceback.format_exception(*sys.exc_info()))
            trace += " in module: %s\n" % module_name
            trace += " recipe: %s\n" % recipe
            raise Exception(trace)
        open("%s/_temp/%s.done" % (CWD, module_name), 'wb').close()
        for pattern in FPATTERNS:
            for fname in glob.glob(pattern):
                if fname not in BADF:
                    shutil.move(fname, "%s__%s" % (dest, fname))
        for pattern in DPATTERNS:
            for dname in glob.glob(pattern):
                shutil.move(dname, dest)
        os.chdir(CWD)
        shutil.rmtree(tmpdir, True)
        sys.stderr.write('Finished with %s\n' % module_name)
    return 0

for path in ['_temp', 'source/cookbook/_static',
             'source/visualizing/colormaps/_static']:
    fpath = os.path.join(CWD, path)
    if os.path.exists(fpath):
        shutil.rmtree(fpath)
    os.makedirs(fpath)

os.chdir('_temp')
recipes = []
for rpath in ['source/cookbook', 'source/visualizing/colormaps']:
    fpath = os.path.join(CWD, rpath)
    sys.path.append(fpath)
    recipes += glob.glob('%s/*.py' % fpath)
WPOOL = Pool(processes=6)
RES = WPOOL.map_async(run_recipe, ((recipe,) for recipe in recipes))
RES.get()
os.chdir(CWD)



import yt
import matplotlib.pyplot as plt
from mpl_toolkits.axes_grid1 import AxesGrid

fns = ['Enzo_64/DD0005/data0005','Enzo_64/DD0015/data0015', 'Enzo_64/DD0025/data0025', 'Enzo_64/DD0035/data0035']

fig = plt.figure()

# See http://matplotlib.org/mpl_toolkits/axes_grid/api/axes_grid_api.html
# These choices of keyword arguments produce a four panel plot with a single
# shared narrow colorbar on the right hand side of the multipanel plot. Axes
# labels are drawn for all plots since we're slicing along different directions
# for each plot.
grid = AxesGrid(fig, (0.075,0.075,0.85,0.85),
                nrows_ncols = (2, 2),
                axes_pad = 0.05,
                label_mode = "L",
                share_all = True,
                cbar_location="right",
                cbar_mode="single",
                cbar_size="3%",
                cbar_pad="0%")

for i, fn in enumerate(fns):
    # Load the data and create a single plot
    ds = yt.load(fn) # load data
    p = yt.ProjectionPlot(ds, 'z', 'density', width=(55, 'Mpccm'))

    # Ensure the colorbar limits match for all plots
    p.set_zlim('density', 1e-4, 1e-2)

    # This forces the ProjectionPlot to redraw itself on the AxesGrid axes.
    plot = p.plots['density']
    plot.figure = fig
    plot.axes = grid[i].axes
    plot.cax = grid.cbar_axes[i]

    # Finally, this actually redraws the plot.
    p._setup_plots()

plt.savefig('multiplot_2x2_time_series.png')



import yt
import numpy as np

ds = yt.load("IsolatedGalaxy/galaxy0030/galaxy0030")

# We start by building a default volume rendering scene

im, sc = yt.volume_render(ds, field=("gas","density"), fname="v0.png", sigma_clip=6.0)

sc.camera.set_width(ds.arr(0.1,'code_length'))
tf = sc.get_source().transfer_function
tf.clear()
tf.add_layers(4, 0.01, col_bounds = [-27.5,-25.5],
        alpha=np.logspace(-3,0,4), colormap = 'RdBu_r')
sc.render()
sc.save("v1.png", sigma_clip=6.0)

# In this case, the default alphas used (np.logspace(-3,0,Nbins)) does not
# accentuate the outer regions of the galaxy. Let's start by bringing up the
# alpha values for each contour to go between 0.1 and 1.0

tf = sc.get_source().transfer_function
tf.clear()
tf.add_layers(4, 0.01, col_bounds = [-27.5,-25.5],
        alpha=np.logspace(0,0,4), colormap = 'RdBu_r')
sc.render()
sc.save("v2.png", sigma_clip=6.0)

# Now let's set the grey_opacity to True.  This should make the inner portions
# start to be obcured

tf.grey_opacity = True
sc.render()
sc.save("v3.png", sigma_clip=6.0)

# That looks pretty good, but let's start bumping up the opacity.

tf.clear()
tf.add_layers(4, 0.01, col_bounds = [-27.5,-25.5],
        alpha=10.0*np.ones(4,dtype='float64'), colormap = 'RdBu_r')
sc.render()
sc.save("v4.png", sigma_clip=6.0)

# Let's bump up again to see if we can obscure the inner contour.

tf.clear()
tf.add_layers(4, 0.01, col_bounds = [-27.5,-25.5],
        alpha=30.0*np.ones(4,dtype='float64'), colormap = 'RdBu_r')
sc.render()
sc.save("v5.png", sigma_clip=6.0)

# Now we are losing sight of everything.  Let's see if we can obscure the next
# layer

tf.clear()
tf.add_layers(4, 0.01, col_bounds = [-27.5,-25.5],
        alpha=100.0*np.ones(4,dtype='float64'), colormap = 'RdBu_r')
sc.render()
sc.save("v6.png", sigma_clip=6.0)

# That is very opaque!  Now lets go back and see what it would look like with
# grey_opacity = False

tf.grey_opacity=False
sc.render()
sc.save("v7.png", sigma_clip=6.0)

# That looks pretty different, but the main thing is that you can see that the
# inner contours are somewhat visible again.



import yt

# Load the dataset
ds = yt.load("SecondOrderTris/RZ_p_no_parts_do_nothing_bcs_cone_out.e", step=-1)

# Create a line plot of the variables 'u' and 'v' with 1000 sampling points evenly spaced
# between the coordinates (0, 0, 0) and (0, 1, 0)
plot = yt.LinePlot(ds, [('all', 'v'), ('all', 'u')], (0, 0, 0), (0, 1, 0), 1000)

# Add a legend
plot.annotate_legend(('all', 'v'))

# Save the line plot
plot.save()



from yt.mods import *
import inspect
from textwrap import TextWrapper

ds = load("RD0005-mine/RedshiftOutput0005")

output = open("source/visualizing/_cb_docstrings.inc", "w")

template = """

.. function:: %(clsname)s%(sig)s:

   (This is a proxy for :class:`~%(clsproxy)s`.)

%(docstring)s

"""

tw = TextWrapper(initial_indent='   ', subsequent_indent='   ', width=60)

def write_docstring(f, name, cls):
    if not hasattr(cls, '_type_name') or cls._type_name is None: return
    for clsi in inspect.getmro(cls):
        docstring = inspect.getdoc(clsi.__init__)
        if docstring is not None: break
    clsname = cls._type_name
    sig = inspect.formatargspec(*inspect.getargspec(cls.__init__))
    sig = sig.replace("**kwargs", "**field_parameters")
    clsproxy = "yt.visualization.plot_modifications.%s" % (cls.__name__)
    #docstring = "\n".join(["   %s" % line for line in docstring.split("\n")])
    #print docstring
    f.write(template % dict(clsname = clsname, sig = sig, clsproxy=clsproxy,
                            docstring = "\n".join(tw.wrap(docstring))))
                            #docstring = docstring))

for n,c in sorted(callback_registry.items()):
    write_docstring(output, n, c)
    print(".. autoclass:: yt.visualization.plot_modifications.%s" % n)
    print("   :members:")
    print()



import yt
import matplotlib.pyplot as plt
from mpl_toolkits.axes_grid1 import AxesGrid

fn = "IsolatedGalaxy/galaxy0030/galaxy0030"
ds = yt.load(fn) # load data

fig = plt.figure()

# See http://matplotlib.org/mpl_toolkits/axes_grid/api/axes_grid_api.html
# These choices of keyword arguments produce a four panel plot that includes
# four narrow colorbars, one for each plot.  Axes labels are only drawn on the
# bottom left hand plot to avoid repeating information and make the plot less
# cluttered.
grid = AxesGrid(fig, (0.075,0.075,0.85,0.85),
                nrows_ncols = (2, 2),
                axes_pad = 1.0,
                label_mode = "1",
                share_all = True,
                cbar_location="right",
                cbar_mode="each",
                cbar_size="3%",
                cbar_pad="0%")

fields = ['density', 'velocity_x', 'velocity_y', 'velocity_magnitude']

# Create the plot.  Since SlicePlot accepts a list of fields, we need only
# do this once.
p = yt.SlicePlot(ds, 'z', fields)

# Velocity is going to be both positive and negative, so let's make these
# slices use a linear colorbar scale
p.set_log('velocity_x', False)
p.set_log('velocity_y', False)

p.zoom(2)

# For each plotted field, force the SlicePlot to redraw itself onto the AxesGrid
# axes.
for i, field in enumerate(fields):
    plot = p.plots[field]
    plot.figure = fig
    plot.axes = grid[i].axes
    plot.cax = grid.cbar_axes[i]

# Finally, redraw the plot on the AxesGrid axes.
p._setup_plots()

plt.savefig('multiplot_2x2.png')



import yt
from matplotlib.animation import FuncAnimation
from matplotlib import rc_context

ts = yt.load('GasSloshingLowRes/sloshing_low_res_hdf5_plt_cnt_*')

plot = yt.SlicePlot(ts[0], 'z', 'density')
plot.set_zlim('density', 8e-29, 3e-26)

fig = plot.plots['density'].figure

# animate must accept an integer frame number. We use the frame number
# to identify which dataset in the time series we want to load
def animate(i):
    ds = ts[i]
    plot._switch_ds(ds)

animation = FuncAnimation(fig, animate, frames=len(ts))

# Override matplotlib's defaults to get a nicer looking font
with rc_context({'mathtext.fontset': 'stix'}):
    animation.save('animation.mp4')



import re
import subprocess
from docutils import statemachine
from sphinx.util.compat import Directive

def setup(app):
    app.add_directive('config_help', GetConfigHelp)
    setup.app = app
    setup.config = app.config
    setup.confdir = app.confdir

    retdict = dict(
        version='1.0',
        parallel_read_safe=True,
        parallel_write_safe=True
    )

    return retdict

class GetConfigHelp(Directive):
    required_arguments = 1
    optional_arguments = 0
    final_argument_whitespace = True

    def run(self):
        rst_file = self.state_machine.document.attributes['source']
        data = subprocess.check_output(
            self.arguments[0].split(" ") + ['-h']).decode('utf8').split('\n')
        ind = next((i for i, val in enumerate(data)
                    if re.match('\s{0,3}\{.*\}\s*$', val)))
        lines = ['.. code-block:: none', ''] + data[ind + 1:]
        self.state_machine.insert_input(
            statemachine.string2lines("\n".join(lines)), rst_file)
        return []



import yt
import numpy as np

# Load the dataset.
ds = yt.load("GasSloshing/sloshing_nomag2_hdf5_plt_cnt_0150")

# Create a slice object
slc = yt.SlicePlot(ds,'x','density',width=(800.0,'kpc'))

# Get a reference to the matplotlib axes object for the plot
ax = slc.plots['density'].axes

# Let's adjust the x axis tick labels
for label in ax.xaxis.get_ticklabels():
    label.set_color('red')
    label.set_fontsize(16)

# Get a reference to the matplotlib figure object for the plot
fig = slc.plots['density'].figure

# And create a mini-panel of a gaussian histogram inside the plot
rect = (0.2,0.2,0.2,0.2)
new_ax = fig.add_axes(rect)

n, bins, patches = new_ax.hist(np.random.randn(1000)+20, 50,
                               facecolor='black', edgecolor='black')

# Make sure its visible
new_ax.tick_params(colors='white')

# And label it
la = new_ax.set_xlabel('Dinosaurs per furlong')
la.set_color('white')

slc.save()



from yt.mods import *
import inspect
from textwrap import TextWrapper

ds = load("RD0005-mine/RedshiftOutput0005")

output = open("source/analyzing/_obj_docstrings.inc", "w")

template = """

.. class:: %(clsname)s%(sig)s:

   For more information, see :ref:`%(docstring)s`
   (This is a proxy for :class:`~%(clsproxy)sBase`.)
"""

tw = TextWrapper(initial_indent='   ', subsequent_indent='   ', width=60)

def write_docstring(f, name, cls):
    for clsi in inspect.getmro(cls):
        docstring = inspect.getdoc(clsi.__init__)
        if docstring is not None: break
    clsname = name
    sig = inspect.formatargspec(*inspect.getargspec(cls.__init__))
    sig = sig.replace("**kwargs", "**field_parameters")
    clsproxy = "yt.data_objects.data_containers.%s" % (cls.__name__)
    f.write(template % dict(clsname = clsname, sig = sig, clsproxy=clsproxy,
                            docstring = 'physical-object-api'))

for n,c in sorted(ds.__dict__.items()):
    if hasattr(c, '_con_args'):
        print(n)
        write_docstring(output, n, c)



import yt

# Load the dataset.
ds = yt.load("GalaxyClusterMerger/fiducial_1to3_b0.273d_hdf5_plt_cnt_0175")

# Create density-weighted projections of temperature (weighted line integrals)

yt.ProjectionPlot(ds, "x", "temperature", weight_field="density").save()
yt.ProjectionPlot(ds, "y", "temperature", weight_field="density").save()
yt.ProjectionPlot(ds, "z", "temperature", weight_field="density").save()



import yt
import numpy as np
from yt.visualization.volume_rendering.api import PointSource
from yt.units import kpc

ds = yt.load('IsolatedGalaxy/galaxy0030/galaxy0030')

sc = yt.create_scene(ds)

np.random.seed(1234567)

npoints = 1000

# Random particle positions
vertices = np.random.random([npoints, 3])*200*kpc

# Random colors
colors = np.random.random([npoints, 4])

# Set alpha value to something that produces a good contrast with the volume
# rendering
colors[:, 3] = 0.1

points = PointSource(vertices, colors=colors)
sc.add_source(points)

sc.camera.width = 300*kpc

sc.save(sigma_clip=5)



import yt
import yt.units as u

ds = yt.load('HiresIsolatedGalaxy/DD0044/DD0044')

center = [0.53, 0.53, 0.53]
normal = [0,0,1]
radius = 40*u.kpc
height = 5*u.kpc

disk = ds.disk(center, [0,0,1], radius, height)

profile = yt.create_profile(
    data_source=disk,
    bin_fields=["radius"],
    fields=["cylindrical_tangential_velocity_absolute"],
    n_bins=256,
    units=dict(radius="kpc",
               cylindrical_tangential_velocity_absolute="km/s"),
    logs=dict(radius=False),
    weight_field='cell_mass',
    extrema=dict(radius=(0,40)),
    )

plot = yt.ProfilePlot.from_profiles(profile)

plot.set_log('cylindrical_tangential_velocity_absolute', False)
plot.set_ylim('cylindrical_tangential_velocity_absolute', 60, 160)

plot.save()



import yt
from yt.visualization.volume_rendering.interactive_vr import \
    SceneGraph, BlockCollection, TrackballCamera
from yt.visualization.volume_rendering.interactive_loop import \
    RenderingContext
from yt.visualization.volume_rendering import glfw_inputhook 

rc = RenderingContext(1280, 960)

scene = SceneGraph()
collection = BlockCollection()

ds = yt.load("IsolatedGalaxy/galaxy0030/galaxy0030")

dd = ds.all_data()
collection.add_data(dd, "density")

scene.add_collection(collection)

position = (1.0, 1.0, 1.0)
c = TrackballCamera(position=position, focus=ds.domain_center,
                    near_plane=0.1)

callbacks = rc.setup_loop(scene, c)
rl = rc(scene, c, callbacks)

# To make this work from IPython execute:
#
# glfw_inputhook.inputhook_manager.enable_gui("glfw", app=rl)



import yt
from yt.utilities.parallel_tools.parallel_analysis_interface \
    import communication_system

import h5py
import time
import numpy as np

@yt.derived_field(name="IonizedHydrogen", units="",
                  display_name=r"\frac{\rho_{HII}}{\rho_H}")
def IonizedHydrogen(field, data):
    return data["HII_Density"]/(data["HI_Density"]+data["HII_Density"])

ts = yt.DatasetSeries("SED800/DD*/*.index", parallel=8)

ionized_z = np.zeros(ts[0].domain_dimensions, dtype="float32")

t1 = time.time()
for ds in ts.piter():
    z = ds.current_redshift
    for g in parallel_objects(ds.index.grids, njobs = 16):
        i1, j1, k1 = g.get_global_startindex() # Index into our domain
        i2, j2, k2 = g.get_global_startindex() + g.ActiveDimensions
        # Look for the newly ionized gas
        newly_ion = ((g["IonizedHydrogen"] > 0.999)
                   & (ionized_z[i1:i2,j1:j2,k1:k2] < z))
        ionized_z[i1:i2,j1:j2,k1:k2][newly_ion] = z
        g.clear_data()

print("Iteration completed  %0.3e" % (time.time()-t1))
comm = communication_system.communicators[-1]
for i in range(ionized_z.shape[0]):
    ionized_z[i,:,:] = comm.mpi_allreduce(ionized_z[i,:,:], op="max")
    print("Slab % 3i has minimum z of %0.3e" % (i, ionized_z[i,:,:].max()))
t2 = time.time()
print("Completed.  %0.3e" % (t2-t1))

if comm.rank == 0:
    f = h5py.File("IonizationCube.h5", "w")
    f.create_dataset("/z", data=ionized_z)



import yt
import numpy as np
from yt.data_objects.particle_filters import add_particle_filter
from matplotlib import pyplot as plt

def formed_star(pfilter, data):
    filter = data["all", "creation_time"] > 0
    return filter

add_particle_filter("formed_star", function=formed_star, filtered_type='all',
                    requires=["creation_time"])

filename = "IsolatedGalaxy/galaxy0030/galaxy0030"

ds = yt.load(filename)
ds.add_particle_filter('formed_star')
ad = ds.all_data()
masses = ad['formed_star', 'particle_mass'].in_units('Msun')
formation_time = ad['formed_star', 'creation_time'].in_units('yr')

time_range = [0, 5e8] # years
n_bins = 1000
hist, bins = np.histogram(formation_time, bins=n_bins, range=time_range,)
inds = np.digitize(formation_time, bins=bins)
time = (bins[:-1] + bins[1:])/2

sfr = np.array([masses[inds == j+1].sum()/(bins[j+1]-bins[j])
                for j in range(len(time))])
sfr[sfr == 0] = np.nan

plt.plot(time/1e6, sfr)
plt.xlabel('Time  [Myr]')
plt.ylabel('SFR  [M$_\odot$ yr$^{-1}$]')
plt.savefig("filter_sfr.png")



import yt

# Load the dataset
ds = yt.load("IsolatedGalaxy/galaxy0030/galaxy0030")

# Create a projection and save it with the default colormap ('algae')
p = yt.ProjectionPlot(ds, "z", "density", width=(100, 'kpc'))
p.save()

# Change the colormap to 'dusk' and save again.  We must specify
# a different filename here or it will save it over the top of
# our first projection.
p.set_cmap(field="density", cmap='dusk')
p.save('proj_with_dusk_cmap.png')

# Change the colormap to 'hot' and save again.
p.set_cmap(field="density", cmap='hot')
p.save('proj_with_hot_cmap.png')



import yt

# Load the dataset
ds = yt.load("GasSloshing/sloshing_nomag2_hdf5_plt_cnt_0150")

# Create density slices of several fields along the x axis
yt.SlicePlot(ds, 'x', ['density','temperature','pressure'],
             width = (800.0, 'kpc')).save()



import subprocess
import sys
from sphinx.util.compat import Directive

def setup(app):
    app.add_directive('yt_showfields', ShowFields)
    setup.app = app
    setup.config = app.config
    setup.confdir = app.confdir
    
    retdict = dict(
        version='1.0',
        parallel_read_safe=True,
        parallel_write_safe=True
    )

    return retdict

class ShowFields(Directive):
    required_arguments = 0
    optional_arguments = 0
    parallel_read_safe = True
    parallel_write_safe = True

    def run(self):
        rst_file = self.state_machine.document.attributes['source']
        lines = subprocess.check_output(
            [sys.executable, './helper_scripts/show_fields.py'])
        lines = lines.decode('utf8')
        lines = lines.split('\n')
        self.state_machine.insert_input(lines, rst_file)
        return []



import yt
from yt.analysis_modules.cosmological_observation.api import \
     LightCone

# Create a LightCone object extending from z = 0 to z = 0.1.

# We have already set up the redshift dumps to be
# used for this, so we will not use any of the time
# data dumps.
lc = LightCone('enzo_tiny_cosmology/32Mpc_32.enzo',
               'Enzo', 0., 0.1,
               observer_redshift=0.0,
               time_data=False)

# Calculate a randomization of the solution.
lc.calculate_light_cone_solution(seed=123456789, filename="LC/solution.txt")

# Choose the field to be projected.
field = 'szy'

# Use the LightCone object to make a projection with a 600 arcminute
# field of view and a resolution of 60 arcseconds.
# Set njobs to -1 to have one core work on each projection
# in parallel.
lc.project_light_cone((600.0, "arcmin"), (60.0, "arcsec"), field,
                      weight_field=None,
                      save_stack=True,
                      save_final_image=True,
                      save_slice_images=True,
                      njobs=-1)

# By default, the light cone projections are kept in the LC directory,
# but this moves them back to the current directory so that they're rendered
# in our cookbook.
import shutil, glob
for file in glob.glob('LC/*png'):
    shutil.move(file, '.')



import yt
from yt.analysis_modules.halo_analysis.halo_catalog import HaloCatalog

# Load the dataset
ds = yt.load("Enzo_64/RD0006/RedshiftOutput0006")

# Load the halo list from a rockstar output for this dataset
halos = yt.load('rockstar_halos/halos_0.0.bin')

# Create the halo catalog from this halo list
hc = HaloCatalog(halos_ds=halos)
hc.load()

# Create a projection with the halos overplot on top
p = yt.ProjectionPlot(ds, "x", "density")
p.annotate_halos(hc)
p.save()



import yt

ds = yt.load("IsolatedGalaxy/galaxy0030/galaxy0030")  # load data

field = "temperature"  # The field to average
weight = "cell_mass"  # The weight for the average

ad = ds.all_data()  # This is a region describing the entire box,
                    # but note it doesn't read anything in yet!

# We now use our 'quantities' call to get the average quantity
average_value = ad.quantities.weighted_average_quantity(field, weight)

print("Average %s (weighted by %s) is %0.3e %s" % (field, weight, average_value, average_value.units))



import yt

# load the dataset
ds = yt.load('IsolatedGalaxy/galaxy0030/galaxy0030')

# create our plot
p = yt.ParticlePlot(ds, 'particle_position_x', 'particle_position_y', color='b')

# zoom in a little bit
p.set_width(500, 'kpc')

#save result
p.save()



import os
import yt
from yt.analysis_modules.cosmological_observation.api import \
    LightRay

# Create a directory for the light rays
if not os.path.isdir("LR"):
    os.mkdir('LR')

# Create a LightRay object extending from z = 0 to z = 0.1
# and use only the redshift dumps.
lr = LightRay("enzo_tiny_cosmology/32Mpc_32.enzo",
              'Enzo', 0.0, 0.1,
              use_minimum_datasets=True,
              time_data=False)

# Make a light ray, and set njobs to -1 to use one core
# per dataset.
lr.make_light_ray(seed=123456789,
                  solution_filename='LR/lightraysolution.txt',
                  data_filename='LR/lightray.h5',
                  fields=['temperature', 'density'],
                  njobs=-1)

# Optionally, we can now overplot the part of this ray that intersects
# one output from the source dataset in a ProjectionPlot
ds = yt.load('enzo_tiny_cosmology/RD0004/RD0004')
p = yt.ProjectionPlot(ds, 'z', 'density')
p.annotate_ray(lr)
p.save()



import matplotlib.pyplot as plt
import yt

# Load the dataset.
ds = yt.load("IsolatedGalaxy/galaxy0030/galaxy0030")

# Create a sphere of radius 1 Mpc centered on the max density location.
sp = ds.sphere("max", (1, "Mpc"))

# Calculate and store the bulk velocity for the sphere.
bulk_velocity = sp.quantities.bulk_velocity()
sp.set_field_parameter('bulk_velocity', bulk_velocity)

# Create a 1D profile object for profiles over radius
# and add a velocity profile.
prof = yt.create_profile(sp, 'radius', ('gas', 'velocity_magnitude'),
                         units = {'radius': 'kpc'},
                         extrema = {'radius': ((0.1, 'kpc'), (1000.0, 'kpc'))},
                         weight_field='cell_mass')

# Create arrays to plot.
radius = prof.x
mean = prof['gas', 'velocity_magnitude']
std = prof.standard_deviation['gas', 'velocity_magnitude']

# Plot the average velocity magnitude.
plt.loglog(radius, mean, label='Mean')
# Plot the variance of the velocity magnitude.
plt.loglog(radius, std, label='Standard Deviation')
plt.xlabel('r [kpc]')
plt.ylabel('v [cm/s]')
plt.legend()

plt.savefig('velocity_profiles.png')



# -*- coding: utf-8 -*-
"""Module for cookbook testing


This test should be run from main yt directory.

Example:

      $ sed -e '/where/d' -i nose.cfg setup.cfg
      $ nosetests doc/source/cookbook/tests/test_cookbook.py -P -v
"""
import glob
import os
import sys
import subprocess


def run_with_capture(*args, **kwargs):
    sp = subprocess.Popen(*args,
                          stdout=subprocess.PIPE,
                          stderr=subprocess.PIPE,
                          **kwargs)
    out, err = sp.communicate()
    if out:
        sys.stdout.write(out.decode("UTF-8"))
    if err:
        sys.stderr.write(err.decode("UTF-8"))

    if sp.returncode != 0:
        retstderr = " ".join(args[0])
        retstderr += "\n\nTHIS IS THE REAL CAUSE OF THE FAILURE:\n" 
        retstderr += err.decode("UTF-8") + "\n"
        raise subprocess.CalledProcessError(sp.returncode, retstderr)

    return sp.returncode


PARALLEL_TEST = {"rockstar_nest.py": "3"}
BLACKLIST = ["opengl_ipython.py", "opengl_vr.py", "matplotlib-animation.py"]

if sys.version_info >= (3,0,0):
    BLACKLIST.append("rockstar_nest.py")

def test_recipe():
    '''Dummy test grabbing all cookbook's recipes'''
    for fname in glob.glob("doc/source/cookbook/*.py"):
        recipe = os.path.basename(fname)
        if recipe in BLACKLIST:
            continue
        check_recipe.description = "Testing recipe: %s" % recipe
        if recipe in PARALLEL_TEST:
            yield check_recipe, \
                ["mpiexec", "-n", PARALLEL_TEST[recipe], "python", fname]
        else:
            yield check_recipe, ["python", fname]


def check_recipe(cmd):
    '''Run single recipe'''
    proc = subprocess.Popen(cmd, stdout=subprocess.PIPE,
                            stderr=subprocess.PIPE)
    out, err = proc.communicate()
    if out:
        sys.stdout.write(out.decode("utf8"))
    if err:
        sys.stderr.write(err.decode("utf8"))

    if proc.returncode != 0:
        retstderr = " ".join(cmd)
        retstderr += "\n\nTHIS IS THE REAL CAUSE OF THE FAILURE:\n" 
        retstderr += err.decode("UTF-8") + "\n"
        raise subprocess.CalledProcessError(proc.returncode, retstderr)



import yt

# Load the dataset.
ds = yt.load("IsolatedGalaxy/galaxy0030/galaxy0030")

# You can create a derived field by manipulating any existing derived fields
# in any way you choose.  In this case, let's just make a simple one:
# thermal_energy_density = 3/2 nkT

# First create a function which yields your new derived field
def thermal_energy_dens(field, data):
    return (3/2)*data['gas', 'number_density'] * data['gas', 'kT']

# Then add it to your dataset and define the units
ds.add_field(("gas", "thermal_energy_density"), units="erg/cm**3", function=thermal_energy_dens)

# It will now show up in your derived_field_list
for i in sorted(ds.derived_field_list):
    print(i)

# Let's use it to make a projection
ad = ds.all_data()
yt.ProjectionPlot(ds, "x", "thermal_energy_density", weight_field="density", width=(200, 'kpc')).save()



import yt

# load the dataset
ds = yt.load('IsolatedGalaxy/galaxy0030/galaxy0030')

# create our plot
p = yt.ParticlePlot(ds, 'particle_position_x', 'particle_position_y', 'particle_mass', width=(0.5, 0.5))

# pick some appropriate units
p.set_axes_unit('kpc')
p.set_unit('particle_mass', 'Msun')

#save result
p.save()



from yt.mods import *
import inspect
from textwrap import TextWrapper

ds = load("RD0005-mine/RedshiftOutput0005")

output = open("source/analyzing/_dq_docstrings.inc", "w")

template = """

.. function:: %(funcname)s%(sig)s:

   (This is a proxy for :func:`~%(funcproxy)s`.)
%(docstring)s

"""

tw = TextWrapper(initial_indent='   ', subsequent_indent='   ', width=60)

def write_docstring(f, name, func):
    docstring = inspect.getdoc(func)
    funcname = name
    sig = inspect.formatargspec(*inspect.getargspec(func))
    sig = sig.replace("data, ", "")
    sig = sig.replace("(data)", "()")
    funcproxy = "yt.data_objects.derived_quantities.%s" % (func.__name__)
    docstring = "\n".join(["   %s" % line for line in docstring.split("\n")])
    f.write(template % dict(funcname = funcname, sig = sig, funcproxy=funcproxy,
                            docstring = docstring))
                            #docstring = "\n".join(tw.wrap(docstring))))

dd = ds.all_data()
for n,func in sorted(dd.quantities.functions.items()):
    print(n, func)
    write_docstring(output, n, func[1])



vals = ["FluidQuantities", "Particles", "Parameters", "Units",
        "ReadOnDemand", "LoadRawData", "LevelOfSupport", "ContactPerson"]

class CodeSupport(object):
    def __init__(self, **kwargs):
        self.support = {}
        for v in vals: self.support[v] = "N"
        for k, v in kwargs.items():
            if k in vals: self.support[k] = v

Y = "Y"
N = "N"

code_names = ["Enzo", "Orion", "FLASH", "RAMSES", "Chombo",
              "Gadget", "ART", "ZEUS"]


codes = dict(
    Enzo = CodeSupport( FluidQuantities = Y,
                        Particles = Y,
                        Parameters = Y,
                        Units = Y,
                        ReadOnDemand = Y,
                        LoadRawData = Y,
                        ContactPerson = "Matt Turk",
                        LevelOfSupport = "Full"),
    Orion = CodeSupport( FluidQuantities = Y,
                         Particles = N,
                         Parameters = Y,
                         Units = Y,
                         ReadOnDemand = Y,
                         LoadRawData = Y,
                         ContactPerson = "Jeff Oishi",
                         LevelOfSupport = "Full"),
    FLASH = CodeSupport( FluidQuantities = Y,
                         Particles = N,
                         Parameters = N,
                         Units = Y,
                         ReadOnDemand = Y,
                         LoadRawData = Y,
                         ContactPerson = "John !ZuHone",
                         LevelOfSupport = "Partial"),
    RAMSES = CodeSupport( FluidQuantities = Y,
                         Particles = N,
                         Parameters = N,
                         Units = N,
                         ReadOnDemand = Y,
                         LoadRawData = Y,
                         ContactPerson = "Matt Turk",
                         LevelOfSupport = "Partial"),
    Chombo = CodeSupport( FluidQuantities = Y,
                         Particles = N,
                         Parameters = N,
                         Units = N,
                         ReadOnDemand = Y,
                         LoadRawData = Y,
                         ContactPerson = "Jeff Oishi",
                         LevelOfSupport = "Partial"),
    Gadget = CodeSupport( FluidQuantities = N,
                         Particles = Y,
                         Parameters = Y,
                         Units = Y,
                         ReadOnDemand = N,
                         LoadRawData = N,
                         ContactPerson = "Chris Moody",
                         LevelOfSupport = "Partial"),
    ART = CodeSupport( FluidQuantities = N,
                         Particles = N,
                         Parameters = N,
                         Units = N,
                         ReadOnDemand = N,
                         LoadRawData = N,
                         ContactPerson = "Matt Turk",
                         LevelOfSupport = "None"),
    ZEUS = CodeSupport( FluidQuantities = N,
                         Particles = N,
                         Parameters = N,
                         Units = N,
                         ReadOnDemand = N,
                         LoadRawData = N,
                         ContactPerson = "Matt Turk",
                         LevelOfSupport = "None")
)

print("|| . ||", end=' ')
for c in code_names:
    print("%s || " % (c), end=' ')
print()

for vn in vals:
    print("|| !%s ||" % (vn), end=' ')
    for c in code_names:
        print("%s || " % (codes[c].support[vn]), end=' ')
    print()




import yt

ds = yt.load('Enzo_64/DD0043/data0043')

sc = yt.create_scene(ds, lens_type='perspective')

source = sc[0]

source.set_field('density')
source.set_log(True)

# Set up the camera parameters: focus, width, resolution, and image orientation
sc.camera.focus = ds.domain_center
sc.camera.resolution = 1024
sc.camera.north_vector = [0, 0, 1]
sc.camera.position = [1.7, 1.7, 1.7]

# You may need to adjust the alpha values to get an image with good contrast.
# For the annotate_domain call, the fourth value in the color tuple is the
# alpha value.
sc.annotate_axes(alpha=.02)
sc.annotate_domain(ds, color=[1, 1, 1, .01])

text_string = "T = {} Gyr".format(float(ds.current_time.to('Gyr')))

# save an annotated version of the volume rendering including a representation
# of the transfer function and a nice label showing the simulation time.
sc.save_annotated("vol_annotated.png", sigma_clip=6,
                  text_annotate=[[(.1, 1.05), text_string]])



import yt

# Load the dataset.
ds = yt.load("GasSloshing/sloshing_nomag2_hdf5_plt_cnt_0150")

# Create density slices in all three axes.
yt.SlicePlot(ds, 'x', "density", width = (800.0, 'kpc')).save()
yt.SlicePlot(ds, 'y', "density", width = (800.0, 'kpc')).save()
yt.SlicePlot(ds, 'z', "density", width = (800.0, 'kpc')).save()




import os
import hglib
import pkg_resources
from email.utils import parseaddr

templates = {"header": r"""<Project xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#" xmlns:rdfs="http://www.w3.org/2000/01/rdf-schema#" xmlns="http://usefulinc.com/ns/doap#" xmlns:foaf="http://xmlns.com/foaf/0.1/" xmlns:admin="http://webns.net/mvcb/">
 <name>The yt Project</name>
 <shortname>yt</shortname>
 <shortdesc>Multi-resolution volumetric analysis</shortdesc>
 <description>yt is a python package for analyzing and visualizing volumetric, multi-resolution data.  Originally developed for astrophysical simulations, it is flexible enough to work with data from other domains such as weather, nuclear engineering, seismology and molecular dynamics.</description>
 <homepage rdf:resource="http://yt-project.org/" />
 <download-page rdf:resource="https://pypi.python.org/pypi/yt/" />
 <download-mirror rdf:resource="http://bitbucket.org/yt_analysis/yt/" />
 <bug-database rdf:resource="http://bitbucket.org/yt_analysis/yt/issues" />
 <programming-language>python</programming-language>
 <programming-language>cython</programming-language>
 <license rdf:resource="http://usefulinc.com/doap/licenses/bsd" />
 """,
 "foaf": r"""<foaf:Person>
     <foaf:name>%(realname)s</foaf:name>
  </foaf:Person>
  """,
  "release": r"""
	<release>
		<Version>
			<name>%(name)s</name>
			<created>%(date)s</created>
			<revision>%(revision)s</revision>
		</Version>
	</release>
   """,
  "footer": r"""
 <repository> 
   <HgRepository>
     <browse rdf:resource='https://bitbucket.org/yt_analysis/yt/src' />
     <location rdf:resource='https://bitbucket.org/yt_analysis/yt' />
   </HgRepository>
 </repository> 
</Project>
"""
}

known_releases = [
    ("0.3"  , "2007-12-17"),
    ("1.0.1", "2008-10-25"),
    ("1.5"  , "2009-11-04"),
    ("1.6"  , "2010-01-22"),
    ("1.6.1", "2010-02-11"),
    ("1.7"  , "2010-06-27"),
]

yt_provider = pkg_resources.get_provider("yt")
yt_path = os.path.dirname(yt_provider.module_path)

name_mappings = {
        # Sometimes things get filtered out by hgchurn pointing elsewhere.
        # So we can add them back in manually
        "andrew.wetzel@yale.edu" : "Andrew Wetzel",
        "df11c@my.fsu.edu": "Daniel Fenn",
        "dnarayan@haverford.edu": "Desika Narayanan",
        "jmtomlinson95@gmail.com": "Joseph Tomlinson",
        "kaylea.nelson@yale.edu": "Kaylea Nelson",
        "tabel@slac.stanford.edu": "Tom Abel",
        "pshriwise": "Patrick Shriwise",
        "jnaiman": "Jill Naiman",
        "gsiisg": "Geoffrey So",
        "dcollins4096@gmail.com": "David Collins",
        "bcrosby": "Brian Crosby",
        "astrugarek": "Antoine Strugarek",
        "AJ": "Allyson Julian",
}

name_ignores = ["convert-repo"]

lastname_sort = lambda a: a.rsplit(None, 1)[-1]

def get_release_tags():
    with hglib.open(yt_path) as c:
        releases = {}
        for name, rev, node, islocal in c.tags():
            if name.startswith("yt-"):
                releases[name] = node
        rr = []
        for name, node in sorted(releases.items()):
            date = c.log(node)[-1][-1]
            rr.append((date, name[3:]))
    rr.sort()
    return [(_[1], _[0].strftime("%Y-%M-%d")) for _ in rr]

def developer_names():
    cmd = hglib.util.cmdbuilder("churn", "-c")
    with hglib.open(yt_path) as c:
        emails = set([])
        for dev in c.rawcommand(cmd).split("\n"):
            if len(dev.strip()) == 0: continue
            emails.add(dev.rsplit(None, 2)[0])
        print("Generating real names for {0} emails".format(len(emails)))
        names = set([])
        for email in sorted(emails):
            if email in name_ignores:
                continue
            if email in name_mappings:
                names.add(name_mappings[email])
                continue
            cset = c.log(revrange="last(author('%s'))" % email)
            if len(cset) == 0:
                print("Error finding {0}".format(email))
                realname = email
            else:
                realname, addr = parseaddr(cset[0][4])
            if realname == '':
                realname = email
            if realname in name_mappings:
                names.add(name_mappings[realname])
                continue
            realname = realname.decode('utf-8')
            realname = realname.encode('ascii', 'xmlcharrefreplace')
            names.add(realname)
    #with open("devs.txt", "w") as f:
    #    for name in sorted(names, key=lastname_sort):
    #        f.write("%s\n" % name)
    devs = list(names)
    devs.sort(key=lastname_sort)
    return devs

def generate_doap():
    dev_names = developer_names()
    with open("doap.xml", "w") as f:
        f.write(templates["header"])
        for dev_name in dev_names:
            f.write("<developer>\n")
            f.write(templates["foaf"] % {'realname': dev_name})
            f.write("</developer>\n")
        for release in known_releases + get_release_tags():
            f.write(templates["release"] % {
                'name': "yt " + release[0], 'revision': release[0], 'date': release[1]}
            )
        f.write(templates["footer"])

if __name__ == "__main__":
    generate_doap()



import os
import glob
import sys
from sys import platform as _platform
from setuptools import setup, find_packages
from setuptools.extension import Extension
from setuptools.command.build_ext import build_ext as _build_ext
from setuptools.command.sdist import sdist as _sdist
from setupext import \
    check_for_openmp, \
    check_for_pyembree, \
    read_embree_location, \
    in_conda_env
from distutils.version import LooseVersion
import pkg_resources


if sys.version_info < (2, 7) or (3, 0) < sys.version_info < (3, 4):
    print("yt currently supports Python 2.7 or versions newer than Python 3.4")
    print("certain features may fail unexpectedly and silently with older "
          "versions.")
    sys.exit(1)

try:
    distribute_ver = \
        LooseVersion(pkg_resources.get_distribution("distribute").version)
    if distribute_ver < LooseVersion("0.7.3"):
        print("Distribute is a legacy package obsoleted by setuptools.")
        print("We strongly recommend that you just uninstall it.")
        print("If for some reason you cannot do it, you'll need to upgrade it")
        print("to latest version before proceeding:")
        print("    pip install -U distribute")
        sys.exit(1)
except pkg_resources.DistributionNotFound:
    pass  # yay!

VERSION = "3.4.1"

if os.path.exists('MANIFEST'):
    os.remove('MANIFEST')

try:
    import pypandoc
    long_description = pypandoc.convert_file('README.md', 'rst')
except (ImportError, IOError):
    with open('README.md') as file:
        long_description = file.read()

if check_for_openmp() is True:
    omp_args = ['-fopenmp']
else:
    omp_args = None

if os.name == "nt":
    std_libs = []
else:
    std_libs = ["m"]

cython_extensions = [
    Extension("yt.analysis_modules.photon_simulator.utils",
              ["yt/analysis_modules/photon_simulator/utils.pyx"],
              include_dirs=["yt/utilities/lib"]),
    Extension("yt.analysis_modules.ppv_cube.ppv_utils",
              ["yt/analysis_modules/ppv_cube/ppv_utils.pyx"],
              libraries=std_libs),
    Extension("yt.geometry.grid_visitors",
              ["yt/geometry/grid_visitors.pyx"],
              include_dirs=["yt/utilities/lib"],
              libraries=std_libs),
    Extension("yt.geometry.grid_container",
              ["yt/geometry/grid_container.pyx"],
              include_dirs=["yt/utilities/lib/"],
              libraries=std_libs),
    Extension("yt.geometry.oct_container",
              ["yt/geometry/oct_container.pyx",
               "yt/utilities/lib/tsearch.c"],
              include_dirs=["yt/utilities/lib"],
              libraries=std_libs),
    Extension("yt.geometry.oct_visitors",
              ["yt/geometry/oct_visitors.pyx"],
              include_dirs=["yt/utilities/lib/"],
              libraries=std_libs),
    Extension("yt.geometry.particle_oct_container",
              ["yt/geometry/particle_oct_container.pyx"],
              include_dirs=["yt/utilities/lib/"],
              libraries=std_libs),
    Extension("yt.geometry.selection_routines",
              ["yt/geometry/selection_routines.pyx"],
              include_dirs=["yt/utilities/lib/"],
              libraries=std_libs),
    Extension("yt.geometry.particle_deposit",
              ["yt/geometry/particle_deposit.pyx"],
              include_dirs=["yt/utilities/lib/"],
              libraries=std_libs),
    Extension("yt.geometry.particle_smooth",
              ["yt/geometry/particle_smooth.pyx"],
              include_dirs=["yt/utilities/lib/"],
              libraries=std_libs),
    Extension("yt.geometry.fake_octree",
              ["yt/geometry/fake_octree.pyx"],
              include_dirs=["yt/utilities/lib/"],
              libraries=std_libs),
    Extension("yt.utilities.lib.autogenerated_element_samplers",
              ["yt/utilities/lib/autogenerated_element_samplers.pyx"],
              include_dirs=["yt/utilities/lib/"]),
    Extension("yt.utilities.lib.bitarray",
              ["yt/utilities/lib/bitarray.pyx"],
              libraries=std_libs),
    Extension("yt.utilities.lib.bounding_volume_hierarchy",
              ["yt/utilities/lib/bounding_volume_hierarchy.pyx"],
              include_dirs=["yt/utilities/lib/"],
              extra_compile_args=omp_args,
              extra_link_args=omp_args,
              libraries=std_libs,
              depends=["yt/utilities/lib/mesh_triangulation.h"]),
    Extension("yt.utilities.lib.contour_finding",
              ["yt/utilities/lib/contour_finding.pyx"],
              include_dirs=["yt/utilities/lib/",
                            "yt/geometry/"],
              libraries=std_libs),
    Extension("yt.utilities.lib.fnv_hash",
              ["yt/utilities/lib/fnv_hash.pyx"],
              include_dirs=["yt/utilities/lib/"],
              libraries=std_libs),
    Extension("yt.utilities.lib.geometry_utils",
              ["yt/utilities/lib/geometry_utils.pyx"],
              extra_compile_args=omp_args,
              extra_link_args=omp_args,
              libraries=std_libs),
    Extension("yt.utilities.lib.marching_cubes",
              ["yt/utilities/lib/marching_cubes.pyx",
               "yt/utilities/lib/fixed_interpolator.c"],
              include_dirs=["yt/utilities/lib/"],
              libraries=std_libs,
              depends=["yt/utilities/lib/fixed_interpolator.h"]),
    Extension("yt.utilities.lib.mesh_triangulation",
              ["yt/utilities/lib/mesh_triangulation.pyx"],
              depends=["yt/utilities/lib/mesh_triangulation.h"]),
    Extension("yt.utilities.lib.pixelization_routines",
              ["yt/utilities/lib/pixelization_routines.pyx",
               "yt/utilities/lib/pixelization_constants.c"],
              include_dirs=["yt/utilities/lib/"],
              libraries=std_libs,
              depends=["yt/utilities/lib/pixelization_constants.h"]),
    Extension("yt.utilities.lib.primitives",
              ["yt/utilities/lib/primitives.pyx"],
              libraries=std_libs),
    Extension("yt.utilities.lib.cosmology_time",
              ["yt/utilities/lib/cosmology_time.pyx"]),
    Extension("yt.utilities.lib.origami",
              ["yt/utilities/lib/origami.pyx",
               "yt/utilities/lib/origami_tags.c"],
              include_dirs=["yt/utilities/lib/"],
              depends=["yt/utilities/lib/origami_tags.h"]),
    Extension("yt.utilities.lib.grid_traversal",
              ["yt/utilities/lib/grid_traversal.pyx",
               "yt/utilities/lib/fixed_interpolator.c"],
              include_dirs=["yt/utilities/lib/"],
              libraries=std_libs,
              depends=["yt/utilities/lib/fixed_interpolator.h"]),
    Extension("yt.utilities.lib.image_samplers",
              ["yt/utilities/lib/image_samplers.pyx",
               "yt/utilities/lib/fixed_interpolator.c"],
              include_dirs=["yt/utilities/lib/"],
              libraries=std_libs,
              extra_compile_args=omp_args,
              extra_link_args=omp_args,
              depends=["yt/utilities/lib/fixed_interpolator.h"]),
    Extension("yt.utilities.lib.partitioned_grid",
              ["yt/utilities/lib/partitioned_grid.pyx",
               "yt/utilities/lib/fixed_interpolator.c"],
              include_dirs=["yt/utilities/lib/"],
              libraries=std_libs,
              depends=["yt/utilities/lib/fixed_interpolator.h"]),
    Extension("yt.utilities.lib.element_mappings",
              ["yt/utilities/lib/element_mappings.pyx"],
              libraries=std_libs),
    Extension("yt.utilities.lib.alt_ray_tracers",
              ["yt/utilities/lib/alt_ray_tracers.pyx"],
              libraries=std_libs),
]

lib_exts = [
    "particle_mesh_operations", "depth_first_octree", "fortran_reader",
    "interpolators", "misc_utilities", "basic_octree", "image_utilities",
    "points_in_volume", "quad_tree", "mesh_utilities",
    "amr_kdtools", "lenses", "distance_queue", "allocation_container"
]
for ext_name in lib_exts:
    cython_extensions.append(
        Extension("yt.utilities.lib.{}".format(ext_name),
                  ["yt/utilities/lib/{}.pyx".format(ext_name)],
                  libraries=std_libs))

lib_exts = ["write_array", "ragged_arrays", "line_integral_convolution"]
for ext_name in lib_exts:
    cython_extensions.append(
        Extension("yt.utilities.lib.{}".format(ext_name),
                  ["yt/utilities/lib/{}.pyx".format(ext_name)]))

extensions = [
    Extension("yt.analysis_modules.halo_finding.fof.EnzoFOF",
              ["yt/analysis_modules/halo_finding/fof/EnzoFOF.c",
               "yt/analysis_modules/halo_finding/fof/kd.c"],
              libraries=std_libs),
    Extension("yt.analysis_modules.halo_finding.hop.EnzoHop",
              glob.glob("yt/analysis_modules/halo_finding/hop/*.c")),
    Extension("yt.frontends.artio._artio_caller",
              ["yt/frontends/artio/_artio_caller.pyx"] +
              glob.glob("yt/frontends/artio/artio_headers/*.c"),
              include_dirs=["yt/frontends/artio/artio_headers/",
                            "yt/geometry/",
                            "yt/utilities/lib/"],
              depends=glob.glob("yt/frontends/artio/artio_headers/*.c")),
]

# EMBREE
if check_for_pyembree() is not None:
    embree_extensions = [
        Extension("yt.utilities.lib.mesh_construction",
                  ["yt/utilities/lib/mesh_construction.pyx"],
                  depends=["yt/utilities/lib/mesh_triangulation.h"]),
        Extension("yt.utilities.lib.mesh_traversal",
                  ["yt/utilities/lib/mesh_traversal.pyx"]),
        Extension("yt.utilities.lib.mesh_samplers",
                  ["yt/utilities/lib/mesh_samplers.pyx"]),
        Extension("yt.utilities.lib.mesh_intersection",
                  ["yt/utilities/lib/mesh_intersection.pyx"]),
    ]

    embree_prefix = os.path.abspath(read_embree_location())
    embree_inc_dir = [os.path.join(embree_prefix, 'include')]
    embree_lib_dir = [os.path.join(embree_prefix, 'lib')]
    if in_conda_env():
        conda_basedir = os.path.dirname(os.path.dirname(sys.executable))
        embree_inc_dir.append(os.path.join(conda_basedir, 'include'))
        embree_lib_dir.append(os.path.join(conda_basedir, 'lib'))

    if _platform == "darwin":
        embree_lib_name = "embree.2"
    else:
        embree_lib_name = "embree"

    for ext in embree_extensions:
        ext.include_dirs += embree_inc_dir
        ext.library_dirs += embree_lib_dir
        ext.language = "c++"
        ext.libraries += std_libs
        ext.libraries += [embree_lib_name]

    cython_extensions += embree_extensions

# ROCKSTAR
if os.path.exists("rockstar.cfg"):
    try:
        rd = open("rockstar.cfg").read().strip()
    except IOError:
        print("Reading Rockstar location from rockstar.cfg failed.")
        print("Please place the base directory of your")
        print("Rockstar install in rockstar.cfg and restart.")
        print("(ex: \"echo '/path/to/Rockstar-0.99' > rockstar.cfg\" )")
        sys.exit(1)

    rockstar_extdir = "yt/analysis_modules/halo_finding/rockstar"
    rockstar_extensions = [
        Extension("yt.analysis_modules.halo_finding.rockstar.rockstar_interface",
                  sources=[os.path.join(rockstar_extdir, "rockstar_interface.pyx")]),
        Extension("yt.analysis_modules.halo_finding.rockstar.rockstar_groupies",
                  sources=[os.path.join(rockstar_extdir, "rockstar_groupies.pyx")])
    ]
    for ext in rockstar_extensions:
        ext.library_dirs.append(rd)
        ext.libraries.append("rockstar")
        ext.define_macros.append(("THREADSAFE", ""))
        ext.include_dirs += [rd,
                             os.path.join(rd, "io"), os.path.join(rd, "util")]
    extensions += rockstar_extensions

if os.environ.get("GPERFTOOLS", "no").upper() != "NO":
    gpd = os.environ["GPERFTOOLS"]
    idir = os.path.join(gpd, "include")
    ldir = os.path.join(gpd, "lib")
    print(("INCLUDE AND LIB DIRS", idir, ldir))
    cython_extensions.append(
        Extension("yt.utilities.lib.perftools_wrap",
                  ["yt/utilities/lib/perftools_wrap.pyx"],
                  libraries=["profiler"],
                  library_dirs=[ldir],
                  include_dirs=[idir]))

class build_ext(_build_ext):
    # subclass setuptools extension builder to avoid importing cython and numpy
    # at top level in setup.py. See http://stackoverflow.com/a/21621689/1382869
    def finalize_options(self):
        try:
            import cython
            import numpy
        except ImportError:
            raise ImportError(
"""Could not import cython or numpy. Building yt from source requires
cython and numpy to be installed. Please install these packages using
the appropriate package manager for your python environment.""")
        if LooseVersion(cython.__version__) < LooseVersion('0.24'):
            raise RuntimeError(
"""Building yt from source requires Cython 0.24 or newer but
Cython %s is installed. Please update Cython using the appropriate
package manager for your python environment.""" %
                cython.__version__)
        if LooseVersion(numpy.__version__) < LooseVersion('1.10.4'):
            raise RuntimeError(
"""Building yt from source requires NumPy 1.10.4 or newer but
NumPy %s is installed. Please update NumPy using the appropriate
package manager for your python environment.""" %
                numpy.__version__)
        from Cython.Build import cythonize
        self.distribution.ext_modules[:] = cythonize(
                self.distribution.ext_modules)
        _build_ext.finalize_options(self)
        # Prevent numpy from thinking it is still in its setup process
        # see http://stackoverflow.com/a/21621493/1382869
        if isinstance(__builtins__, dict):
            # sometimes this is a dict so we need to check for that
            # https://docs.python.org/3/library/builtins.html
            __builtins__["__NUMPY_SETUP__"] = False
        else:
            __builtins__.__NUMPY_SETUP__ = False
        self.include_dirs.append(numpy.get_include())

class sdist(_sdist):
    # subclass setuptools source distribution builder to ensure cython
    # generated C files are included in source distribution.
    # See http://stackoverflow.com/a/18418524/1382869
    # subclass setuptools source distribution builder to ensure cython
    # generated C files are included in source distribution and readme
    # is converted from markdown to restructured text.  See
    # http://stackoverflow.com/a/18418524/1382869
    def run(self):
        # Make sure the compiled Cython files in the distribution are
        # up-to-date

        try:
            import pypandoc
        except ImportError:
            raise RuntimeError(
                'Trying to create a source distribution without pypandoc. '
                'The readme will not render correctly on pypi without '
                'pypandoc so we are exiting.'
            )
        # Make sure the compiled Cython files in the distribution are up-to-date
        from Cython.Build import cythonize
        cythonize(cython_extensions)
        _sdist.run(self)

setup(
    name="yt",
    version=VERSION,
    description="An analysis and visualization toolkit for volumetric data",
    long_description = long_description,
    classifiers=["Development Status :: 5 - Production/Stable",
                 "Environment :: Console",
                 "Intended Audience :: Science/Research",
                 "License :: OSI Approved :: BSD License",
                 "Operating System :: MacOS :: MacOS X",
                 "Operating System :: POSIX :: AIX",
                 "Operating System :: POSIX :: Linux",
                 "Programming Language :: C",
                 "Programming Language :: Python :: 2",
                 "Programming Language :: Python :: 2.7",
                 "Programming Language :: Python :: 3",
                 "Programming Language :: Python :: 3.4",
                 "Programming Language :: Python :: 3.5",
                 "Topic :: Scientific/Engineering :: Astronomy",
                 "Topic :: Scientific/Engineering :: Physics",
                 "Topic :: Scientific/Engineering :: Visualization"],
    keywords='astronomy astrophysics visualization ' +
    'amr adaptivemeshrefinement',
    entry_points={'console_scripts': [
        'yt = yt.utilities.command_line:run_main',
    ],
        'nose.plugins.0.10': [
            'answer-testing = yt.utilities.answer_testing.framework:AnswerTesting'
    ]
    },
    packages=find_packages(),
    include_package_data = True,
    install_requires=[
        'matplotlib>=1.5.3',
        'setuptools>=19.6',
        'sympy>=1.0',
        'numpy>=1.10.4',
        'IPython>=1.0',
    ],
    extras_require = {
        'hub':  ["girder_client"]
    },
    cmdclass={'sdist': sdist, 'build_ext': build_ext},
    author="The yt project",
    author_email="yt-dev@python.org",
    url="http://yt-project.org/",
    license="BSD 3-Clause",
    zip_safe=False,
    scripts=["scripts/iyt"],
    ext_modules=cython_extensions + extensions,
    python_requires='>=2.7,!=3.0.*,!=3.1.*,!=3.2.*,!=3.3.*'
)



import yt

# Load the data file.
ds = yt.load("Sedov_3d/sedov_hdf5_chk_0002")

# Make a traditional slice plot.
sp = yt.SlicePlot(ds, "x", "density")

# Overlay the slice plot with thick red contours of density.
sp.annotate_contour("density", ncont=3, clim=(1e-2,1e-1), label=True,
                    plot_args={"colors": "red",
                               "linewidths": 2})

# What about some nice temperature contours in blue?
sp.annotate_contour("temperature", ncont=3, clim=(1e-8,1e-6), label=True,
                    plot_args={"colors": "blue",
                               "linewidths": 2})

# This is the plot object.
po = sp.plots["density"]

# Turn off the colormap image, leaving just the contours.
po.axes.images[0].set_visible(False)

# Remove the colorbar and its label.
po.figure.delaxes(po.figure.axes[1])

# Save it and ask for a close fit to get rid of the space used by the colorbar.
sp.save(mpl_kwargs={'bbox_inches':'tight'})



import yt

# load the dataset
ds = yt.load('IsolatedGalaxy/galaxy0030/galaxy0030')

# create our plot
p = yt.ParticlePlot(ds, 'particle_position_x', 'particle_velocity_z', ['particle_mass'])

# pick some appropriate units
p.set_unit('particle_position_x', 'Mpc')
p.set_unit('particle_velocity_z', 'km/s')
p.set_unit('particle_mass', 'Msun')

# save result
p.save()



import yt
import numpy as np
from yt.data_objects.particle_filters import add_particle_filter


# Define filter functions for our particle filters based on stellar age.
# In this dataset particles in the initial conditions are given creation
# times arbitrarily far into the future, so stars with negative ages belong
# in the old stars filter.
def stars_10Myr(pfilter, data):
    age = data.ds.current_time - data["Stars", "creation_time"]
    filter = np.logical_and(age >= 0, age.in_units('Myr') < 10)
    return filter

def stars_100Myr(pfilter, data):
    age = (data.ds.current_time - data["Stars", "creation_time"]).in_units('Myr')
    filter = np.logical_and(age >= 10, age < 100)
    return filter

def stars_old(pfilter, data):
    age = data.ds.current_time - data["Stars", "creation_time"]
    filter = np.logical_or(age < 0, age.in_units('Myr') >= 100)
    return filter

# Create the particle filters
add_particle_filter("stars_young", function=stars_10Myr, filtered_type='Stars',
                    requires=["creation_time"])
add_particle_filter("stars_medium", function=stars_100Myr, filtered_type='Stars',
                    requires=["creation_time"])
add_particle_filter("stars_old", function=stars_old, filtered_type='Stars',
                    requires=["creation_time"])

# Load a dataset and apply the particle filters
filename = "TipsyGalaxy/galaxy.00300"
ds = yt.load(filename)
ds.add_particle_filter('stars_young')
ds.add_particle_filter('stars_medium')
ds.add_particle_filter('stars_old')

# What are the total masses of different ages of star in the whole simulation
# volume?
ad = ds.all_data()
mass_young = ad['stars_young', 'particle_mass'].in_units('Msun').sum()
mass_medium = ad['stars_medium', 'particle_mass'].in_units('Msun').sum()
mass_old = ad['stars_old', 'particle_mass'].in_units('Msun').sum()
print("Mass of young stars = %g Msun" % mass_young)
print("Mass of medium stars = %g Msun" % mass_medium)
print("Mass of old stars = %g Msun" % mass_old)

# Generate 4 projections: gas density, young stars, medium stars, old stars
fields = [('gas', 'density'), ('deposit', 'stars_young_cic'),
          ('deposit', 'stars_medium_cic'), ('deposit', 'stars_old_cic')]

prj = yt.ProjectionPlot(ds, 'z', fields, center="max", width=(100, 'kpc'))
prj.save()



import yt

# Load the dataset.
ds = yt.load("IsolatedGalaxy/galaxy0030/galaxy0030")

# Create a 15 kpc radius sphere, centered on the center of the sim volume
sp = ds.sphere("center", (15.0, "kpc"))

# Get the angular momentum vector for the sphere.
L = sp.quantities.angular_momentum_vector()

print("Angular momentum vector: {0}".format(L))

# Create an OffAxisProjectionPlot of density centered on the object with the L
# vector as its normal and a width of 25 kpc on a side
p = yt.OffAxisProjectionPlot(ds, L, "density", sp.center, (25, "kpc"))
p.save()



header = """Cookbook
========

yt scripts can be a bit intimidating, and at times a bit obtuse.  But there's a
lot you can do, and this section of the manual will assist with figuring out
how to do some fairly common tasks -- which can lead to combining these, with
other Python code, into more complicated and advanced tasks.

.. note::
   All of these scripts are located in the mercurial repository at
   http://bitbucket.org/yt_analysis/cookbook/

"""
footer = """ """

import cStringIO, sys, glob, os
from mercurial import hg, ui, commands
uii = ui.ui()

if "--index" in sys.argv:
    recipes = open("source/cookbook/recipes.rst", "w")
else:
    recipes = cStringIO.StringIO()
recipes.write(header)

url = "here: http://bitbucket.org/yt_analysis/cookbook/raw/tip/%s ."

def cond_output(f, v):
    if not v:
        f.write(".. rubric:: Sample Output\n\n")
    return True

repo = hg.repository(uii, "../cookbook/")
commands.pull(uii, repo, "http://bitbucket.org/yt_analysis/cookbook/")
ctx = repo["tip"]
for file in ctx:
    if not file.startswith("recipes/"): continue
    print("Parsing %s" % (file))
    lines = ctx[file].data().split("\n")
    fn = file[8:-3]
    title = fn.replace("_", " ").capitalize()
    title += "\n" + "-" * len(title) + "\n"*2
    title = ".. _cookbook-%s:\n\n%s" % (fn, title)
    if lines[0] != '"""':
        print("    Bad docstring: breaking.")
        print(file)
    di = lines[1:].index('"""')
    docstring = lines[1:di+1]
    recipe = lines[di+2:]
    recipes.write(".. include:: %s.inc\n" % fn)
    output = open("source/cookbook/%s.inc" % fn, "w")
    output.write(title)
    output.write("\n".join(docstring))
    output.write("\n")
    output.write("\nThe latest version of this recipe can be downloaded ")
    output.write(url % file)
    output.write("\n\n.. code-block:: python\n\n")
    for line in recipe:
        output.write("   " + line + "\n")
    if os.path.isdir("../cookbook/images/%s" % (fn)):
        output.write("\n")
        ndir = "source/cookbook/_%s/" % (fn)
        if not os.path.isdir(ndir): os.mkdir(ndir)
        written = False
        for ifn in sorted(glob.glob("../cookbook/images/%s/*.png" % (fn))):
            written = cond_output(output, written)
            ofn = "%s/%s_%s" % (ndir, fn, os.path.basename(ifn))
            open(ofn, "wb").write(open(ifn, "rb").read())
            output.write(".. image:: _%s/%s_%s\n" % (fn, fn, os.path.basename(ifn)) +
                         "   :width: 240\n" +
                         "   :target: ../_images/%s_%s\n" % (fn, os.path.basename(ifn))
                        )
        for ifn in sorted(glob.glob("../cookbook/images/%s/*.txt" % (fn))):
            written = cond_output(output, written)
            open("%s/%s_%s" % (ndir, fn, os.path.basename(ifn)), "w").write(open(ifn, "r").read())
            output.write("\n``%s``\n\n" % os.path.basename(ifn))
            output.write(".. literalinclude:: _%s/%s_%s\n" % (fn, fn, os.path.basename(ifn)))
    output.write("\n\n")
    output.close()

recipes.write(footer)
recipes.close()



import yt
from yt.visualization.volume_rendering.interactive_vr import \
    SceneGraph, BlockCollection, TrackballCamera
from yt.visualization.volume_rendering.interactive_loop import \
    RenderingContext

ds = yt.load("IsolatedGalaxy/galaxy0030/galaxy0030")

# Create GLUT window
rc = RenderingContext(1280, 960)

# Create a 3d Texture from all_data()
collection = BlockCollection()
dd = ds.all_data()
collection.add_data(dd, "density")

# Initiliaze basic Scene and pass the data
scene = SceneGraph()
scene.add_collection(collection)

# Create default camera
position = (1.0, 1.0, 1.0)
c = TrackballCamera(position=position, focus=ds.domain_center,
                    near_plane=0.1)

# Start rendering loop
rc.start_loop(scene, c)
